function [imSegmented imContour] = segmentimage(imOriginal, blkSize, enh, smooth)

%[imSegmented imContour] = segmentimage(imOriginal, blkSize, enh, smooth)
%
% Funkce segmentuje obrazek na otisk a pozadi. Vypocet probiha po blocich.
%
%   imOriginal - vstupni otisk
%   blkSize - velikost bloku
%   enh - vylepseni kontrastu, TRUE pro zapnuto, FALSE pro vypnuto
%   smooth - velikost vyhlazovaciho okna
%
% vystupy: 
%   imSegmented - binarni obrazek stejne velikosti jako imOriginal, 1
%       oznacuje otisk a 0 znaci pozadi
%   imContour - kontura segmentace binarni obrazek obsahujici pouze
%   konturu, ne celou plochu segmentace

if nargin < 2
    blkSize = 16;
end
if nargin < 3
    enh = true;
end
if nargin < 4
    smooth = 16;
end
    
warning off MATLAB:conversionToLogical

im = imOriginal;

width = size(im, 2);
height = size(im, 1);
im = im/max(max(im));
im = blkpad(im, blkSize); % pad to a multiple of blkSize to avoid zero padding

if enh % enhance contrast
    im = histeq(im);
    im = medfilt2(im);
end

imSegmented = blkproc(im, [blkSize blkSize], @processblk);

% resize to original size because of the padding
imSegmented = imcrop(imSegmented, [0 0 width height]);


% remove isolated blocks
bw = bwareaopen(imSegmented, 12*(blkSize^2), 4);
bw = imcomplement(bw);
bw = bwareaopen(bw, 12*(blkSize^2), 4);
bw = imcomplement(bw);


% smoothing
bw = padarray(bw, [blkSize blkSize], 'symmetric', 'both');
bw = medfilt2(bw, [smooth smooth]);
bw = bw(blkSize + 1 : size(bw,1) - blkSize, blkSize + 1 : size(bw,2) - blkSize);


imContour = bwperim(bw);
imContour = imresize(imContour, size(imOriginal));

imSegmented = bw;
imSegmented = imresize(imSegmented, size(imOriginal));






%--------------------------------------------------------------------------
%Sub-Function processblk
%--------------------------------------------------------------------------
function  imout = processblk(imin) 

[fx, fy] = gradient(double(imin));

gradDev = (std2(fx) + std2(fy));




if gradDev > 30   % 30 good for IPS images
    imout =  ones(size(imin));
else
    imout = zeros(size(imin));
end



